package info.aspirecn.cloud.yysj.dao.es;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.ObjectMapper;
import info.aspirecn.cloud.yysj.config.IndexConfig;
import info.aspirecn.cloud.yysj.model.entity.GatewayFirstLog;
import info.aspirecn.cloud.yysj.model.entity.GatewayFirstLogList;
import info.aspirecn.cloud.yysj.model.enums.QueryType;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.rest.RestStatus;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * ElasticSearch 云验数据平台子订单索引查询
 *
 * @author dingliqiang
 */
@Repository
public class YysjGatewayFirstIndex {

    /**
     * 车辆查询
     */
    private static final String QUERY_CAR_INFO = "/info_check/carInfo.do";

    /**
     * 驾驶员查询
     */
    private static final String QUERY_DRIVER_INFO = "/info_check/driverInfo.do";
    /**
     * 运单查询
     */
    private static final String QUERY_WAYBILL_INFO = "/info_check/waybill.do";

    /**
     * 查询产品类型
     */
    private static final String[] QUERY_TYPE = {QUERY_CAR_INFO, QUERY_DRIVER_INFO, QUERY_WAYBILL_INFO};

    @Autowired
    private RestHighLevelClient restHighLevelClient;

    @Autowired
    private IndexConfig indexConfig;

    @Autowired
    private ObjectMapper objectMapper;

    /**
     * 获取用户主订单列表
     * @param userId 云验平台用户ID
     * @param orderId 主订单ID
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param form 开始条数
     * @param size 一次查询大小
     * @return 主订单列表
     */
    @SneakyThrows(IOException.class)
    public GatewayFirstLogList getFirstLogList(String userId, String orderId, String startTime, String endTime
            , int form, int size) {
        // 设置返回结果集合
        List<GatewayFirstLog> gatewayFirstLogList = new ArrayList<>();
        // 验证时间
        if (StringUtils.isEmpty(startTime)){
           startTime = "now-10/d";
        }
        if (StringUtils.isEmpty(endTime)){
            startTime = "now";
        }
        // 构建查询条件
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery()
                .filter(QueryBuilders.rangeQuery("DateTime").gte(startTime).lte(endTime).format("yyyy-MM-dd"))
                .must(QueryBuilders.termQuery("MG-UserId", userId))
                .must(QueryBuilders.termsQuery("URI.keyword", QUERY_TYPE));
        // 判断条件有无订单ID号
        if (StringUtils.isNotEmpty(orderId)){
            queryBuilder.must(QueryBuilders.termQuery("traceId.keyword", orderId));
        }
        // 构建查询源构建器
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.from(form);
        searchSourceBuilder.size(size);
        searchSourceBuilder.sort("DateTime", SortOrder.DESC);
        searchSourceBuilder.query(queryBuilder);
        // 创建查询请求对象，将查询对象配置到其中
        SearchRequest request = new SearchRequest(indexConfig.getIndexYysjFirst()).source(searchSourceBuilder);
        // 执行查询请求，获取响应信息
        SearchResponse response = restHighLevelClient.search(request, RequestOptions.DEFAULT);
        if (!RestStatus.OK.equals(response.status()) || response.getAggregations() == null) {
            SearchHits hits = response.getHits();
            for (SearchHit hit : hits) {
                // 将 JSON 转换成对象
                GatewayFirstLog firstLog = JSON.parseObject(hit.getSourceAsString(), GatewayFirstLog.class);
                // 转换对象中的查询类型
                if (firstLog.getType().equalsIgnoreCase(QUERY_CAR_INFO)){
                    firstLog.setType(QueryType.QUERY_CAR_INFO.getName());
                }else if (firstLog.getType().equalsIgnoreCase(QUERY_DRIVER_INFO)){
                    firstLog.setType(QueryType.QUERY_DRIVER_INFO.getName());
                }else if (firstLog.getType().equalsIgnoreCase(QUERY_WAYBILL_INFO)){
                    firstLog.setType(QueryType.QUERY_WAYBILL_INFO.getName());
                }
                // 加入到对象集合
                gatewayFirstLogList.add(firstLog);
            }
        }

        return new GatewayFirstLogList().setGatewayFirstLogs(gatewayFirstLogList).setTotal(response.getHits().totalHits);
    }

}
